/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include <assert.h>
#include <unistd.h>
#include <iostream>
#include <thread>
#include "xunfeispeechengine.h"

void writeBinaryDataToFile(const std::string &filename,
                           const std::vector<char> &data) {
    std::ofstream outputFile(filename,
                             std::ios::out | std::ios::binary | std::ios::app);
    if (!outputFile.is_open()) {
        std::cerr << "Failed to open file for writing." << std::endl;
        return;
    }

    outputFile.write(data.data(), data.size());

    if (!outputFile.good()) {
        std::cerr << "Error occurred while writing to file." << std::endl;
    } else {
        std::cout << "Binary data has been written to file: " << filename
                  << std::endl;
    }

    outputFile.close();
}

std::vector<char> readAudioData(const std::string &filePath) {
    std::ifstream file(filePath, std::ios::binary);
    if (!file.is_open()) {
        std::cerr << "Failed to open file: " << filePath << std::endl;
        return {};
    }

    file.seekg(0, std::ios::end);
    std::streampos fileSize = file.tellg();
    file.seekg(0, std::ios::beg);
    std::vector<char> audioData(fileSize);
    file.read(audioData.data(), fileSize);
    return audioData;
}

std::vector<std::vector<char>>
splitAudioData(const std::vector<char> &audioData, size_t chunkSize) {
    std::vector<std::vector<char>> chunks;
    size_t dataSize = audioData.size();
    size_t numChunks = dataSize / chunkSize;
    size_t remainder = dataSize % chunkSize;

    for (size_t i = 0; i < numChunks; ++i) {
        chunks.push_back(
            std::vector<char>(audioData.begin() + i * chunkSize,
                              audioData.begin() + (i + 1) * chunkSize));
    }

    if (remainder != 0) {
        chunks.push_back(
            std::vector<char>(audioData.end() - remainder, audioData.end()));
    }

    return chunks;
}

void testSpeech() {
    XunfeiSpeechEngine *engine = new XunfeiSpeechEngine();

    const std::string configJson =
        u8R"({"appId":"fakeAppId", "apiKey":"fakeApiKey", "secretKey":"fakeSecretKey"})";
    engine->setConfig(configJson);

    auto audioData = readAudioData("16k_10.pcm");

    ai_engine::lm::EngineError error;
    auto ret = engine->initRecognizeOnceModule(error);
    assert(ret == true);

    engine->setRecognizingCallback(
        [](ai_engine::lm::speech::RecognitionResult result) {
            std::cout << "MID_TEXT: " << result.text << std::endl;
        });
    engine->setRecognizedCallback(
        [](ai_engine::lm::speech::RecognitionResult result) {
            std::cout << "FIN_TEXT: " << result.text << std::endl;
        });

    ret = engine->recognizeOnce("\{\"rate\": 16000}", audioData, error);
    std::cout << "recongnize once result: " << error.getMessage() << std::endl;
    assert(ret == true);
    ret = engine->destroyRecognizeOnceModule(error);

    ret = engine->initContinuousRecognitionModule(error);
    assert(ret == true);

    ret = engine->startContinuousRecognition("\{\"rate\": 16000}", error);
    assert(ret == true);
    auto audioDatas = splitAudioData(audioData, 2560);
    for (auto &data : audioDatas) {
        ret = engine->writeContinuousRecognitionAudioData(data, error);
        assert(ret == true);
        usleep(80 * 1000);
    }
    sleep(5);
    engine->stopContinuousRecognition(error);
    engine->destroyContinuousRecognitionModule(error);

    ret = engine->initSynthesizeOnceModule(error);
    assert(ret == true);
    std::string text =
        "看起来像是随机敲击键盘产生的字符组合，没有明确的含义或上下文。\
        它可能是一个错误输入、代码片段的一部分，或者仅仅是随机文本。\
        如果您能提供更多的上下文或信息，我可能能更好地理解它的含义或用途。";

    engine->setSynthesizingCallback(
        [](ai_engine::lm::speech::SynthesisResult result) {
            std::cout << result.audioData.size() << std::endl;
            writeBinaryDataToFile("test.mp3", result.audioData);
        });
    engine->setSynthesizedCallback([](ai_engine::lm::speech::ResultType type) {
        std::cout << "Synthesize finished..." << std::endl;
    });

    ret = engine->synthesizeOnce("\{\"speed\": 5}", text, error);
    std::cout << "synthesize once result: " << error.getMessage() << std::endl;
    assert(ret == true);
    ret = engine->destroySynthesizeOnceModule(error);
    assert(ret == true);
}

int main(int argc, char *argv[]) {
    testSpeech();
    return 0;
}